#![allow(non_snake_case)]

use wasm_bindgen::{prelude::*, JsCast};
use web_sys::{HtmlDialogElement, HtmlFormElement, HtmlInputElement};
use yew::prelude::*;
use yewdux::prelude::*;

use crate::components::states::MainState;

use super::create_close_button;

pub fn input_json_input_callback(
    main_dispatch: Dispatch<MainState>,
    modal: HtmlDialogElement,
) -> Closure<dyn FnMut(Event)> {
    Closure::<dyn FnMut(_)>::new(move |e: Event| {
        if let Some(input) = e.current_target() {
            let input = input
                .dyn_into::<HtmlInputElement>()
                .expect("We know this is an input.");
            if let Some(files) = input.files() {
                if let Some(file) = files.get(0) {
                    let fr = web_sys::FileReader::new().unwrap();
                    fr.read_as_text(&file).unwrap();
                    let fr1 = fr.clone(); // Clone to avoid outliving closure
                    let main_dispatch = main_dispatch.clone(); // Clone to satisfy FnMut
                                                               // trait
                    let modal = modal.clone();
                    let onload = Closure::<dyn FnMut(_)>::new(move |_: Event| {
                        if let Some(value) = &fr1.result().ok().and_then(|v| v.as_string()) {
                            let ms = serde_json::from_str::<MainState>(value);
                            match ms {
                                Ok(ms) => main_dispatch.set(ms),
                                Err(e) => log::debug!("{:?}", e),
                            };
                            modal.close();
                        }
                    });
                    fr.set_onload(Some(onload.as_ref().unchecked_ref()));
                    onload.forget(); // Magic (don't touch)
                }
            }
        }
    })
}

pub fn import_json_button_callback(main_dispatch: Dispatch<MainState>) -> Callback<MouseEvent> {
    Callback::from(move |_| {
        let window = web_sys::window().unwrap();
        let document = window.document().unwrap();
        let body = document.body().unwrap();
        let modal = document
            .create_element("dialog")
            .unwrap()
            .dyn_into::<HtmlDialogElement>()
            .unwrap();
        modal.set_text_content(Some("Import File:"));
        let onclose_callback = {
            let modal = modal.clone();
            Closure::<dyn FnMut(_)>::new(move |_: Event| {
                modal.remove();
            })
        };
        modal.set_onclose(Some(onclose_callback.as_ref().unchecked_ref()));

        let close_button = create_close_button(&onclose_callback);
        onclose_callback.forget();
        modal.append_child(&close_button).unwrap();

        let form = document
            .create_element("form")
            .unwrap()
            .dyn_into::<HtmlFormElement>()
            .unwrap();
        let input = document
            .create_element("input")
            .unwrap()
            .dyn_into::<HtmlInputElement>()
            .unwrap();
        input.set_type("file");
        input.set_accept(".json");
        form.append_child(&input).unwrap();

        let input_callback = {
            let main_dispatch = main_dispatch.clone();
            let modal = modal.clone();
            input_json_input_callback(main_dispatch, modal)
        };
        input.set_onchange(Some(input_callback.as_ref().unchecked_ref()));
        input_callback.forget(); // Magic straight from the docs, don't touch :(

        modal.append_child(&form).unwrap();
        body.append_child(&modal).unwrap();
        modal.show_modal().unwrap();
    })
}
