#![allow(non_snake_case)]
use plate_tool_lib::util::num_to_letters;
use wasm_bindgen::prelude::Closure;
use wasm_bindgen::JsCast;
use yew::prelude::*;

use plate_tool_lib::plate::PlateType;
use plate_tool_lib::plate_instances::PlateInstance;
use yewdux::functional::use_store;

// use super::destination_plate::DestinationPlate;
// use super::source_plate::SourcePlate;
use super::plate::Plate;
use crate::components::states::MainState;

#[derive(Properties, PartialEq)]
pub struct PlateContainerProps {
    pub source_dims: Option<PlateInstance>,
    pub destination_dims: Option<PlateInstance>,
}

#[function_component]
pub fn PlateContainer(props: &PlateContainerProps) -> Html {
    let (main_state, _) = use_store::<MainState>();

    let cell_height = {
        let height = web_sys::window()
            .unwrap()
            .inner_height()
            .unwrap()
            .as_f64()
            .unwrap();
        let width = web_sys::window()
            .unwrap()
            .inner_width()
            .unwrap()
            .as_f64()
            .unwrap();
        if let (Some(src_d), Some(dest_d)) = (&props.source_dims, &props.destination_dims) {
            let h = (0.78 * height) / (src_d.plate.size().0 + dest_d.plate.size().0) as f64;
            let w = (0.90 * width) / (src_d.plate.size().1 + dest_d.plate.size().1) as f64;
            f64::min(w, h)
        } else {
            1f64
        }
    };

    let resize_trigger = use_force_update();
    let onresize = Closure::<dyn FnMut(_)>::new(move |_: Event| {
        resize_trigger.force_update();
    });
    web_sys::window()
        .unwrap()
        .set_onresize(Some(onresize.as_ref().unchecked_ref()));
    onresize.forget(); // Magic!

    let heatmap_enabled = main_state.preferences.volume_heatmap;
    let show_current_coordinates = main_state.preferences.show_current_coordinates;
    // (Row, Col)
    let current_coordinates = use_state(|| (0, 0));
    let send_coordinates = {
        if show_current_coordinates {
            let current_coordinates = current_coordinates.clone();
            Some(Callback::from(move |coords| {
                current_coordinates.set(coords)
            }))
        } else {
            None
        }
    };

    html! {
        <div class="plate_container">
            <div class="plate_container--upper-left" >
            if heatmap_enabled {
            <div class="plate_container--heatmap-notice" >
            <h3>{"Volume Heatmap Enabled"}</h3>
            </div>
            }
            if show_current_coordinates {
            <div class="plate_container--location">
            <h3>{format!("R:{} C:{}",
                num_to_letters(current_coordinates.0).unwrap_or("".to_string()),
                current_coordinates.1)}</h3>
            </div>
            }
        </div>
            if let Some(spi) = props.source_dims.clone() {
            if let Some(dpi) = props.destination_dims.clone() {
            <div class="plate_container--source">
                <h2>{spi.name.clone()}</h2>
                <h2>{"Source"}</h2>
                <Plate source_plate={spi.clone()} destination_plate={dpi.clone()}
                cell_height={cell_height} ptype={PlateType::Source}
                send_coordinates={send_coordinates.clone()}
                />
            </div>
            <div class="plate_container--destination">
                <h2>{dpi.name.clone()}</h2>
                <h2>{"Destination"}</h2>
                <Plate source_plate={spi.clone()} destination_plate={dpi.clone()}
                cell_height={cell_height} ptype={PlateType::Destination} send_coordinates={send_coordinates.clone()}/>
            </div>
            } else {
                <h2>{"No Destination Plate Selected"}</h2>
            }
            } else {
                <h2>{"No Source Plate Selected"}</h2>
            }
        </div>
    }
}
